﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Runtime.InteropServices;
using System.IO;
using System.Reflection;
using System.Text;
using System.ComponentModel;

namespace TrustedApp
{
    public static class WIN32CONST
    {
        public const int MAX_INTERFACE_NAME_LEN = 256;
        public const int MAXLEN_PHYSADDR = 8;
        public const int MAXLEN_IFDESCR = 256;
        public const int ANY_SIZE = 1;

        public const int ERROR_INSUFFICIENT_BUFFER = 122;

        public const int CP_ACP = 0;
        public const int CP_UTF8 = 65001;

        public const int MB_PRECOMPOSED = 0X001;
    }

    /// <summary>
    /// 网络类型
    /// </summary>
    public enum NetTypeEnum : int
    {
        Other = 1, //其他
        Ethernet = 6, //以太网
        TokenRing = 9, //令牌环网
        PPP = 23,
        SoftwareLoopBack = 24,
        ATM = 37,
        IEEE80211 = 71,
        Tunnel = 131,
        IEEE1394 = 144
    }

    public enum NetStateEnum : int
    {
        NotOperational = 0, //LAN不可用
        Operational = 1, //WAN未连接
        Disconnected = 2,//LAN未连接 WAN无可用网络
        Connecting = 3,//WAN正在连接
        Connected = 4,//WAN已连接
        Unreachable = 5//LAN默认状态
    };

    [StructLayout( LayoutKind.Sequential, CharSet=CharSet.Unicode)]
    public class MIB_IFROW
    {
        /// <summary>
        /// 接口名称
        /// </summary>
        [MarshalAs( UnmanagedType.ByValTStr, SizeConst= WIN32CONST.MAX_INTERFACE_NAME_LEN)]
        public string wszName;
        /// <summary>
        /// 接口索引，此索引号并非固定不变，当重启连接后索引号可能发生变化
        /// </summary>
        public Int32 dwIndex;
        /// <summary>
        /// 接口类型(IANA)，对应NetTypeEnum枚举
        /// </summary>
        public Int32 dwType;
        /// <summary>
        /// 最大传输单位
        /// </summary>
        public Int32 dwMtu;
        /// <summary>
        /// 每秒传输速度(bits)
        /// </summary>
        public Int32 dwSpeed;
        /// <summary>
        /// 指示bPhysAddr字段的字节长度
        /// </summary>
        public Int32 dwPhysAddrLen;
        /// <summary>
        /// 物理地址
        /// </summary>
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = WIN32CONST.MAXLEN_PHYSADDR)]
        public Byte[] bPhysAddr;
        /// <summary>
        /// 
        /// </summary>
        public Int32 dwAdminStatus;
        /// <summary>
        /// 操作状态
        /// </summary>
        public Int32 dwOperStatus;
        /// <summary>
        /// 从计算机最后一次重启开始的以百分之一妙为单位的时间长度
        /// </summary>
        public Int32 dwLastChange;
        /// <summary>
        /// 总接收数据
        /// </summary>
        public Int32 dwInOctets;
        public Int32 dwInUcastPkts;
        public Int32 dwInNUcastPkts;
        public Int32 dwInDiscards;
        /// <summary>
        /// 接收到的错误数据
        /// </summary>
        public Int32 dwInErrors;
        public Int32 dwInUnknownProtos;
        /// <summary>
        /// 总发出数据
        /// </summary>
        public Int32 dwOutOctets;
        public Int32 dwOutUcastPkts;
        public Int32 dwOutNUcastPkts;
        public Int32 dwOutDiscards;
        /// <summary>
        /// 发出的错误数据
        /// </summary>
        public Int32 dwOutErrors;
        public Int32 dwOutQLen;
        public Int32 dwDescrLen;
        /// <summary>
        /// 接口描述
        /// </summary>
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = WIN32CONST.MAXLEN_IFDESCR)]
        public Byte[] bDescr;
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
    public struct MIB_IFTABLE
    {
        public Int32 dwNumEntries;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = WIN32CONST.ANY_SIZE)]
        public MIB_IFROW[] table;
    }

    public class NetInfo : INotifyPropertyChanged
    {
        private String name;
        public String Name
        {
            get { return name; }
            set
            {
                if (name != value)
                {
                    name = value;
                    OnPropertyChanged("Name");
                }
            }
        }

        private String desc;
        public String Desc
        {
            get { return desc; }
            set
            {
                if (desc != value)
                {
                    desc = value;
                    OnPropertyChanged("Desc");
                }
            }
        }

        private NetTypeEnum type;
        public NetTypeEnum Type
        {
            get { return type; }
            set
            {
                if (type != value)
                {
                    type = value;
                    OnPropertyChanged("Type");
                }
            }
        }

        private NetStateEnum state;
        public NetStateEnum State
        {
            get { return state; }
            set
            {
                if (state != value)
                {
                    state = value;
                    OnPropertyChanged("State");
                }
            }
        }

        private int speed;
        public int Speed
        {
            get { return speed; }
            set
            {
                if (speed != value)
                {
                    speed = value;
                    OnPropertyChanged("Speed");
                }
            }
        }

        private int inSpeed;
        public int InSpeed
        {
            get { return inSpeed; }
            set
            {
                if (inSpeed != value)
                {
                    inSpeed = value;
                    OnPropertyChanged("InSpeed");
                }
            }
        }

        private int outSpeed;
        public int OutSpeed
        {
            get { return outSpeed; }
            set
            {
                if (outSpeed != value)
                {
                    outSpeed = value;
                    OnPropertyChanged("OutSpeed");
                }
            }
        }


        private int inOctets;
        public int InOctets
        {
            get { return inOctets; }
            set
            {
                if (inOctets != value)
                {
                    inOctets = value;
                    OnPropertyChanged("InOctets");
                }
            }
        }

        private int outOctets;
        public int OutOctets
        {
            get { return outOctets; }
            set
            {
                if (outOctets != value)
                {
                    outOctets = value;
                    OnPropertyChanged("OutOctets");
                }
            }
        }

        public void CopyFromIFRow(MIB_IFROW row)
        {
            Name = row.wszName;
            Desc = WIN32API.AsciiToUTF8(row.bDescr);
            Type = (NetTypeEnum)row.dwType;
            State = (NetStateEnum)row.dwOperStatus;
            Speed = row.dwSpeed;
            InOctets = row.dwInOctets / 1024;
            OutOctets = row.dwOutOctets / 1024;
        }

        public static NetInfo ConvertFromIFRow(MIB_IFROW row)
        {
            if (row == null)
                return null;
            Byte[] tmpBytes = Encoding.Convert(Encoding.UTF8, Encoding.Unicode, row.bDescr);
            NetInfo netInfo = new NetInfo()
            {
                Name = row.wszName,
                Desc = WIN32API.AsciiToUTF8(row.bDescr),
                Type = (NetTypeEnum)row.dwType,
                State = (NetStateEnum)row.dwOperStatus,
                Speed = row.dwSpeed,
                InOctets = row.dwInOctets / 1024,
                OutOctets = row.dwOutOctets / 1024
            };
            return netInfo;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }

    public static class WIN32API
    {
        [DllImport("kernel32.dll")]
        public static extern bool GetComputerName(StringBuilder computerName, out Int32 nameLength);

        [DllImport("Iphlpapi.dll")]
        private static extern Int32 GetIfTable(Byte[] pIfTable, out long pdwSize, bool bOrder);

        [DllImport("kernel32.dll")]
        private static extern  int MultiByteToWideChar(uint CodePage,Int32 dwFlags,byte[] lpMultiByteStr,Int32 cbMultiByte,byte[] lpWideCharStr, int cchWideChar);

        public static MIB_IFTABLE GetIfTable()
        {
            MIB_IFTABLE table = new MIB_IFTABLE();
            long size = 0;
            GetIfTable(null, out size, false);
            if (size != 0)
            {
                Byte[] data = new Byte[size];
                long ret = GetIfTable(data, out size, false);
                if (ret == 0)
                {
                    MemoryStream ms = new MemoryStream(data);
                    ms.Position = 0;
                    BinaryReader br = new BinaryReader(ms, Encoding.Unicode);

                    table.dwNumEntries = br.ReadInt32();
                    table.table = new MIB_IFROW[table.dwNumEntries];
                    for (int i = 0; i < table.dwNumEntries; i++)
                    {
                        table.table[i] = new MIB_IFROW();
                        MIB_IFROW curRow = table.table[i];
                        FieldInfo[] fis = typeof(MIB_IFROW).GetFields();
                        foreach (FieldInfo fi in fis)
                        {
                            if (!fi.IsStatic)
                            {
                                MarshalAsAttribute[] attrs = fi.GetCustomAttributes(typeof(MarshalAsAttribute), true) as MarshalAsAttribute[];
                                if (attrs != null && attrs.Length > 0 && attrs[0].SizeConst != 0)
                                {

                                    if (fi.FieldType == typeof(String))
                                    {
                                        Byte[] tmpChars = br.ReadBytes(attrs[0].SizeConst*2);
                                        fi.SetValue(curRow, Encoding.Unicode.GetString(tmpChars, 0, tmpChars.Length).TrimEnd('\0'));
                                    }
                                    else if (fi.FieldType == typeof(Byte[]))
                                    {
                                        fi.SetValue(curRow, br.ReadBytes(attrs[0].SizeConst));
                                    }
                                }
                                else
                                {
                                    if (fi.FieldType == typeof(Int32))
                                    {
                                        Int32 tmpValue = br.ReadInt32();
                                        fi.SetValue(curRow, tmpValue);
                                    }
                                }

                            }
                        }
                    }
                }
            }

            return table;
        }

        public static String AsciiToUTF8(Byte[] asciiBytes)
        {
            int mustBytes = MultiByteToWideChar(WIN32CONST.CP_ACP, WIN32CONST.MB_PRECOMPOSED, asciiBytes, -1, null, 0);
            if (mustBytes > 0)
            {
                Byte[] tmpBytes = new Byte[mustBytes*2];
                if (MultiByteToWideChar(WIN32CONST.CP_ACP, WIN32CONST.MB_PRECOMPOSED, asciiBytes, -1, tmpBytes, mustBytes) != 0)
                {
                   return Encoding.Unicode.GetString(tmpBytes, 0, tmpBytes.Length);
                }
            }
            return "";
        }
    }
}
